@@ -77,7 +77,7 @@ class Token:
 
 
 @pydantic.dataclasses.dataclass
-class TournamentUrl:
+class ItemUrl:
     uid: str
     url: str
 
@@ -100,55 +100,101 @@ async def get_db_op():
 DbOperator = typing.Annotated[db.Operator, fastapi.Depends(get_db_op)]
 
 
+def get_member_uid_from_session(request: fastapi.Request) -> str:
+    if not request.session.get("user_id", None):
+        anonymous_session(request)
+        raise LoginRequired()
+    return request.session["user_id"]
+
+
+#: Check we're in an authenticated session and return the member UID
+MemberUidFromSession = typing.Annotated[
+    str, fastapi.Depends(get_member_uid_from_session)
+]
+
+
+async def get_member_from_session(
+    request: fastapi.Request, member_uid: MemberUidFromSession, op: DbOperator
+) -> models.Person:
+    member = await op.get_member(member_uid, cls=models.Person)
+    # Valid user_id in session, but member not in DB
+    if not member:
+        anonymous_session(request)
+        raise LoginRequired()
+    return member
+
+
+#: Check we're in an authenticated session and return the member data from DB
+PersonFromSession = typing.Annotated[
+    models.Person, fastapi.Depends(get_member_from_session)
+]
+
+
 async def get_tournament(
     op: DbOperator,
     uid: typing.Annotated[str, fastapi.Path(title="Tournament unique ID")],
+    member: PersonFromSession,
 ) -> models.Tournament:
-    return await op.get_tournament(uid)
+    tournament = await op.get_tournament(uid)
+    check_can_admin_tournament(member, tournament)
+    return tournament
 
 
 async def get_tournament_config(
     op: DbOperator,
     uid: typing.Annotated[str, fastapi.Path(title="Tournament unique ID")],
 ) -> models.Tournament:
-    return await op.get_tournament(uid, models.TournamentConfig)
+    return await op.get_tournament(uid, cls=models.TournamentConfig)
 
 
 async def get_tournament_info(
     op: DbOperator,
     uid: typing.Annotated[str, fastapi.Path(title="Tournament unique ID")],
 ) -> models.Tournament:
-    return await op.get_tournament(uid, models.TournamentInfo)
+    return await op.get_tournament(uid, cls=models.TournamentInfo)
 
 
+# the only one that locks, always use that for updates
 async def get_tournament_orchestrator(
     op: DbOperator,
     uid: typing.Annotated[str, fastapi.Path(title="Tournament unique ID")],
 ) -> engine.TournamentOrchestrator:
-    ret = await op.get_tournament(uid, engine.TournamentOrchestrator)
+    ret = await op.get_tournament(uid, True, engine.TournamentOrchestrator)
     if not ret:
         raise fastapi.HTTPException(fastapi.status.HTTP_404_NOT_FOUND)
     return ret
 
 
+# Check the member (from their token) can administrate the tournament
 Tournament = typing.Annotated[models.Tournament, fastapi.Depends(get_tournament)]
-TournamentConfig = typing.Annotated[
-    models.TournamentConfig, fastapi.Depends(get_tournament_config)
-]
+# Check there is a member token (not public) or filter data (players names)
 TournamentInfo = typing.Annotated[
     models.TournamentConfig, fastapi.Depends(get_tournament_info)
 ]
+# This is public information
+TournamentConfig = typing.Annotated[
+    models.TournamentConfig, fastapi.Depends(get_tournament_config)
+]
+# Locks tournament for update
 TournamentOrchestrator = typing.Annotated[
     engine.TournamentOrchestrator, fastapi.Depends(get_tournament_orchestrator)
 ]
 
 
 # ################################################################################ Utils
-async def vekn_sync(tournament: models.Tournament, rounds: int):
-    if not tournament.extra.get("vekn_id"):
-        await vekn.upload_tournament(tournament, rounds)
-    if tournament.state == models.TournamentState.FINISHED:
-        await vekn.upload_tournament_result(tournament)
+async def vekn_sync(tournament: models.Tournament, rounds: int, user: models.Person):
+    if not user.vekn:
+        raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
+    try:
+        if not tournament.extra.get("vekn_id"):
+            await vekn.upload_tournament(tournament, rounds, user.vekn)
+        if tournament.state == models.TournamentState.FINISHED:
+            await vekn.upload_tournament_result(tournament)
+    except vekn.NoVEKN:
+        raise fastapi.HTTPException(
+            fastapi.status.HTTP_503_SERVICE_UNAVAILABLE,
+            detail="VEKN is not available at the moment, please try again later.",
+        )
 
 
 def async_timed_cache(duration: datetime.timedelta = datetime.timedelta(minutes=5)):
@@ -202,9 +248,7 @@ def check_can_change_role(
     match role:
         case (
             models.MemberRole.ADMIN
-            | models.MemberRole.JUDGE
-            | models.MemberRole.ANC_JUDGE
-            | models.MemberRole.NEO_JUDGE
+            | models.MemberRole.RULEMONGER
             | models.MemberRole.ETHICS
             | models.MemberRole.NC
             | models.MemberRole.PTC
@@ -214,20 +258,24 @@ def check_can_change_role(
         case models.MemberRole.PRINCE:
             if models.MemberRole.ADMIN in initiator_roles:
                 return
-            elif (
+            if (
                 models.MemberRole.NC in initiator_roles
                 and member.country == target.country
             ):
                 return
-            else:
-                raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
+            raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
         case models.MemberRole.PLAYTESTER:
             if models.MemberRole.ADMIN in initiator_roles:
                 return
-            elif models.MemberRole.PTC in initiator_roles:
+            if models.MemberRole.PTC in initiator_roles:
                 return
-            else:
-                raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
+            raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
+        case models.MemberRole.JUDGE | models.MemberRole.JUDGEKIN:
+            if models.MemberRole.ADMIN in initiator_roles:
+                return
+            if models.MemberRole.RULEMONGER in initiator_roles:
+                return
+            raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
 
 
 def can_organize(member: models.Person) -> bool:
@@ -238,7 +286,14 @@ def can_organize(member: models.Person) -> bool:
     }
 
 
-def check_can_admin_tournament(member: models.Person, tournament: models.Tournament):
+def check_organizer(member: models.Person) -> None:
+    if not can_organize(member):
+        raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
+
+
+def check_can_admin_tournament(
+    member: models.Person, tournament: models.TournamentConfig
+):
     if models.MemberRole.ADMIN in member.roles:
         return
     if models.MemberRole.NC in member.roles and member.country == tournament.country:
@@ -248,9 +303,14 @@ def check_can_admin_tournament(member: models.Person, tournament: models.Tournam
     raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
 
 
-def check_organizer(member: models.Person) -> None:
-    if not can_organize(member):
-        raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
+def check_can_admin_league(member: models.Person, league: models.League):
+    if models.MemberRole.ADMIN in member.roles:
+        return
+    if models.MemberRole.NC in member.roles and member.country == league.country:
+        return
+    if member.uid in [j.uid for j in league.organizers]:
+        return
+    raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
 
 
 def check_can_change_info(member: models.Person, target: models.Person):
@@ -294,6 +354,7 @@ def check_can_sanction(member: models.Person):
     member_roles = set(member.roles)
     if member_roles & {
         models.MemberRole.ADMIN,
+        models.MemberRole.RULEMONGER,
         models.MemberRole.JUDGE,
         models.MemberRole.ETHICS,
     }:
@@ -376,36 +437,6 @@ SessionContext = typing.Annotated[
 class LoginRequired(Exception): ...
 
 
-def get_member_uid_from_session(request: fastapi.Request) -> str:
-    if not request.session.get("user_id", None):
-        anonymous_session(request)
-        raise LoginRequired()
-    return request.session["user_id"]
-
-
-#: Check we're in an authenticated session and return the member UID
-MemberUidFromSession = typing.Annotated[
-    str, fastapi.Depends(get_member_uid_from_session)
-]
-
-
-async def get_member_from_session(
-    request: fastapi.Request, member_uid: MemberUidFromSession, op: DbOperator
-) -> models.Person:
-    member = await op.get_member(member_uid, cls=models.Person)
-    # Valid user_id in session, but member not in DB
-    if not member:
-        anonymous_session(request)
-        raise LoginRequired()
-    return member
-
-
-#: Check we're in an authenticated session and return the member data from DB
-PersonFromSession = typing.Annotated[
-    models.Person, fastapi.Depends(get_member_from_session)
-]
-
-
 # ############################################################### Security: Social Login
 async def discord_login(
     request: fastapi.Request,
@@ -474,18 +505,10 @@ def _hash_password(password: str) -> str:
 
 
 def set_member_password(member: models.Member, password: str):
-    logging.warning("setting: %s", password)
     member.password_hash = _hash_password(password)
-    logging.warning("hash: %s", member.password_hash)
 
 
 def check_member_password(member: models.Member, password: str) -> bool:
-    logging.warning(
-        "recorded: %s, provided: %s, provided_hash: %s",
-        member.password_hash,
-        password,
-        _hash_password(password),
-    )
     return hmac.compare_digest(member.password_hash, _hash_password(password))
 
 
@@ -563,8 +586,7 @@ oauth2_scheme = fastapi.security.OAuth2AuthorizationCodeBearer(
 )
 
 
-# cache expected auth tokens in memory
-# TODO: remove the keys after an hour has passed
+# cache expected auth tokens in memory - this is reset on every reboot (eg. daily)
 EXPECTED_AUTH_TOKENS = set()
 
 
@@ -646,6 +668,7 @@ def get_member_uid_from_token(
     return user_id
 
 
+# A lightweight uid-only info (no DB query)
 MemberUidFromToken = typing.Annotated[str, fastapi.Depends(get_member_uid_from_token)]
 
 
@@ -663,6 +686,7 @@ async def get_person_from_token(
     return member
 
 
+# The "normal" way of getting a member data (no lock)
 PersonFromToken = typing.Annotated[
     models.Person, fastapi.Depends(get_person_from_token)
 ]
@@ -682,6 +706,7 @@ async def get_member_from_token(
     return member
 
 
+# Get the complete Member data for update (locks)
 MemberFromToken = typing.Annotated[
     models.Member, fastapi.Depends(get_member_from_token)
 ]
