@@ -125,7 +125,7 @@ async def html_get_auth_email_reset(
 ):
     if uid:
         return fastapi.responses.RedirectResponse(
-            request.url_for("member_display", uid=uid)
+            request.url_for("member_display", uid=uid).include_query_params(reset=True)
         )
     else:
         request.session["message"] = "Email verification URL is invalid or outdated"
@@ -246,24 +246,24 @@ def _ranked(members: list[models.Person], category: models.RankingCategoy):
 async def _get_rankings(
     op: dependencies.DbOperator,
 ) -> dict[str, list[tuple[int, models.Person]]]:
-    members = await op.get_ranked_members()
-    return {
-        category.value: list(_ranked(members, category))
-        for category in models.RankingCategoy
-    }
+    ret = {}
+    for category in models.RankingCategoy:
+        members = await op.get_ranked_members(category)
+        ret[category.value] = list(_ranked(members, category))
+    return ret
 
 
 @dependencies.async_timed_cache()
 async def _get_rankings_anonymised(
     op: dependencies.DbOperator,
 ) -> dict[str, list[tuple[int, models.Person]]]:
-    members = await op.get_ranked_members()
-    for member in members:
-        member.name = ""
-    return {
-        category.value: list(_ranked(members, category))
-        for category in models.RankingCategoy
-    }
+    ret = {}
+    for category in models.RankingCategoy:
+        members = await op.get_ranked_members(category)
+        for member in members:
+            member.name = ""
+        ret[category.value] = list(_ranked(members, category))
+    return ret
 
 
 @router.get("/index.html")
@@ -332,7 +332,7 @@ def _filter(cls, filter_cls, obj):
 async def tournament_display(
     request: fastapi.Request,
     context: dependencies.SessionContext,
-    tournament: dependencies.Tournament,
+    tournament: dependencies.TournamentInfo,
 ):
     request.session["next"] = str(
         request.url_for("tournament_display", uid=tournament.uid)
@@ -405,9 +405,7 @@ async def tournament_console(
     request: fastapi.Request,
     context: dependencies.SessionContext,
     tournament: dependencies.Tournament,
-    member: dependencies.PersonFromSession,
 ):
-    dependencies.check_can_admin_tournament(member, tournament)
     context["tournament"] = tournament
     return TEMPLATES.TemplateResponse(
         request=request,
@@ -421,9 +419,7 @@ async def tournament_checkin(
     request: fastapi.Request,
     context: dependencies.SessionContext,
     tournament: dependencies.Tournament,
-    member: dependencies.PersonFromSession,
 ):
-    dependencies.check_can_admin_tournament(member, tournament)
     context["name"] = tournament.name
     context["code"] = tournament.checkin_code
     return TEMPLATES.TemplateResponse(
@@ -439,9 +435,7 @@ async def tournament_print_seating(
     context: dependencies.SessionContext,
     tournament: dependencies.Tournament,
     round: typing.Annotated[int, fastapi.Query()],
-    member: dependencies.PersonFromSession,
 ):
-    dependencies.check_can_admin_tournament(member, tournament)
     context["round_number"] = round
     context["round"] = []
     if round == len(tournament.rounds) and tournament.finals_seeds:
@@ -470,9 +464,7 @@ async def tournament_print_standings(
     request: fastapi.Request,
     context: dependencies.SessionContext,
     tournament: dependencies.Tournament,
-    member: dependencies.PersonFromSession,
 ):
-    dependencies.check_can_admin_tournament(member, tournament)
     context["round_number"] = len(tournament.rounds)
     standings = engine.standings(tournament)
     finished = tournament.state == models.TournamentState.FINISHED
@@ -560,6 +552,7 @@ async def member_display(
     member: dependencies.PersonFromSession,
     op: dependencies.DbOperator,
     uid: typing.Annotated[str, fastapi.Path()],
+    reset: typing.Annotated[bool, fastapi.Query()] | None = None,
 ):
     if member.uid != uid and not member.vekn:
         target = await op.get_member(uid)
@@ -569,3 +562,47 @@ async def member_display(
         name="member/display.html.j2",
         context=context,
     )
+
+
+@router.get("/league/{uid}/display.html")
+async def league_display(
+    request: fastapi.Request,
+    context: dependencies.SessionContext,
+    _: dependencies.MemberUidFromSession,
+    uid: typing.Annotated[str, fastapi.Path()],
+):
+    request.session["next"] = str(request.url_for("league_display", uid=uid))
+    context["league_uid"] = uid
+    return TEMPLATES.TemplateResponse(
+        request=request,
+        name="league/display.html.j2",
+        context=context,
+    )
+
+
+@router.get("/league/list.html")
+async def league_list(
+    request: fastapi.Request,
+    context: dependencies.SessionContext,
+):
+    request.session["next"] = str(request.url_for("league_list"))
+    return TEMPLATES.TemplateResponse(
+        request=request,
+        name="league/list.html.j2",
+        context=context,
+    )
+
+
+@router.get("/league/create.html")
+async def league_create(
+    request: fastapi.Request,
+    context: dependencies.SessionContext,
+    member: dependencies.PersonFromSession,
+):
+    request.session["next"] = str(request.url_for("league_create"))
+    dependencies.check_organizer(member)
+    return TEMPLATES.TemplateResponse(
+        request=request,
+        name="league/display.html.j2",
+        context=context,
+    )
