@@ -24,6 +24,10 @@ SITE_URL_BASE = os.getenv("SITE_URL_BASE", "http://127.0.0.1:8000")
 LOG = logging.getLogger()
 
 
+class NoVEKN(RuntimeError):
+    """Raised when VEKN is not configured or the login/password are not set."""
+
+
 async def get_token(session: aiohttp.ClientSession) -> str:
     # http POST https://www.vekn.net/api/vekn/login -f username=<USER> password=<PWD>
     async with session.post(
@@ -35,7 +39,7 @@ async def get_token(session: aiohttp.ClientSession) -> str:
             return result["data"]["auth"]
         except KeyError:
             LOG.exception("VEKN authentication failure")
-            raise
+            raise NoVEKN()
 
 
 ADMINS = {
@@ -47,17 +51,17 @@ ADMINS = {
     "1002480",
 }
 JUDGES = {
-    "8180022": models.MemberRole.JUDGE,
-    "3200188": models.MemberRole.JUDGE,
+    "8180022": models.MemberRole.RULEMONGER,
+    "3200188": models.MemberRole.RULEMONGER,
     "3190007": models.MemberRole.JUDGE,
-    "4200005": models.MemberRole.JUDGE,
+    "4200005": models.MemberRole.RULEMONGER,
     "8530107": models.MemberRole.JUDGE,
     "2340000": models.MemberRole.JUDGE,
     "6260014": models.MemberRole.JUDGE,
     "1940030": models.MemberRole.JUDGE,
     "1003731": models.MemberRole.JUDGE,
-    "1003455": models.MemberRole.JUDGE,
-    "3200340": models.MemberRole.JUDGE,
+    "1003455": models.MemberRole.RULEMONGER,
+    "3200340": models.MemberRole.RULEMONGER,
     "1003030": models.MemberRole.JUDGE,
     "3070069": models.MemberRole.JUDGE,
     "4960027": models.MemberRole.JUDGE,
@@ -68,28 +72,28 @@ JUDGES = {
     "9510021": models.MemberRole.JUDGE,
     "3370036": models.MemberRole.JUDGE,
     "1000629": models.MemberRole.JUDGE,
-    "1002855": models.MemberRole.ANC_JUDGE,
-    "3340152": models.MemberRole.ANC_JUDGE,
+    "1002855": models.MemberRole.JUDGEKIN,
+    "3340152": models.MemberRole.JUDGEKIN,
     "5360022": models.MemberRole.JUDGE,
-    "8390001": models.MemberRole.NEO_JUDGE,
-    "3070006": models.MemberRole.ANC_JUDGE,
-    "4960046": models.MemberRole.NEO_JUDGE,
-    "6140001": models.MemberRole.NEO_JUDGE,
-    "3020044": models.MemberRole.ANC_JUDGE,
-    "3020010": models.MemberRole.NEO_JUDGE,
-    "1003584": models.MemberRole.ANC_JUDGE,
-    "1003214": models.MemberRole.NEO_JUDGE,
-    "4110004": models.MemberRole.ANC_JUDGE,
-    "4110113": models.MemberRole.ANC_JUDGE,
-    "4100033": models.MemberRole.ANC_JUDGE,
-    "2331000": models.MemberRole.ANC_JUDGE,
-    "3680057": models.MemberRole.ANC_JUDGE,
-    "4100008": models.MemberRole.NEO_JUDGE,
-    "3120101": models.MemberRole.ANC_JUDGE,
-    "4960000": models.MemberRole.NEO_JUDGE,
-    "3010501": models.MemberRole.NEO_JUDGE,
-    "6060022": models.MemberRole.ANC_JUDGE,
-    "5540005": models.MemberRole.NEO_JUDGE,
+    "8390001": models.MemberRole.JUDGEKIN,
+    "3070006": models.MemberRole.JUDGEKIN,
+    "4960046": models.MemberRole.JUDGEKIN,
+    "6140001": models.MemberRole.JUDGEKIN,
+    "3020044": models.MemberRole.JUDGEKIN,
+    "3020010": models.MemberRole.JUDGEKIN,
+    "1003584": models.MemberRole.JUDGEKIN,
+    "1003214": models.MemberRole.JUDGEKIN,
+    "4110004": models.MemberRole.JUDGEKIN,
+    "4110113": models.MemberRole.JUDGEKIN,
+    "4100033": models.MemberRole.JUDGEKIN,
+    "2331000": models.MemberRole.JUDGEKIN,
+    "3680057": models.MemberRole.JUDGEKIN,
+    "4100008": models.MemberRole.JUDGEKIN,
+    "3120101": models.MemberRole.JUDGEKIN,
+    "4960000": models.MemberRole.JUDGEKIN,
+    "3010501": models.MemberRole.JUDGEKIN,
+    "6060022": models.MemberRole.JUDGEKIN,
+    "5540005": models.MemberRole.JUDGEKIN,
     "3530067": models.MemberRole.JUDGE,
 }
 
@@ -476,8 +480,6 @@ def _member_from_vekn_data(data: dict[str, str]) -> models.Member:
     judge_role = JUDGES.get(data["veknid"], None)
     if judge_role:
         roles.append(judge_role)
-    # TODO maybe keep track of the sponsor/recruit relationship
-    # since we're dropping prince and nc prefixes
     return models.Member(
         vekn=data["veknid"],
         name=(data["firstname"] + " " + data["lastname"]).strip(),
@@ -597,6 +599,15 @@ async def get_event(
         if data["venue_id"]:
             venue_data = await get_venue(session, token, data["venue_id"])
         ret = _tournament_from_vekn_data(data, members, venue_data)
+    elif (
+        datetime.datetime.fromisoformat(data["event_startdate"]).date()
+        > datetime.date.today()
+    ):
+        LOG.info("Incoming Event #%s: %s", num, data)
+        venue_data = {}
+        if data["venue_id"]:
+            venue_data = await get_venue(session, token, data["venue_id"])
+        ret = _tournament_from_vekn_data(data, members, venue_data)
     del data
     del result
     return ret
@@ -682,6 +693,9 @@ def _tournament_from_vekn_data(
         proxies=bool(rank == models.TournamentRank.BASIC),
         judges=judges,
     )
+    if not data["players"]:
+        ret.state = models.TournamentState.REGISTRATION
+        return ret
     for idx, pdata in enumerate(data["players"], 1):
         member = members.get(pdata["veknid"])
         if not member:
@@ -858,7 +872,9 @@ TOURNAMENT_TYPE_TO_FORMAT_RANK = {
 }
 
 
-async def upload_tournament(tournament: models.Tournament, rounds: int) -> None:
+async def upload_tournament(
+    tournament: models.Tournament, rounds: int, user_vekn: str
+) -> None:
     type = TournamentType.UNSANCTIONED_TOURNAMENT
     match (tournament.format, tournament.rank):
         case (models.TournamentFormat.Draft, _):
@@ -889,7 +905,7 @@ async def upload_tournament(tournament: models.Tournament, rounds: int) -> None:
     if tournament.finish:
         finish = tournament.finish.astimezone(tz=datetime.timezone.utc)
     else:
-        finish = start + datetime.timedelta(hours=6)
+        finish = start + datetime.timedelta(minutes=1)
     try:
         async with aiohttp.ClientSession() as session:
             token = await get_token(session)
@@ -917,10 +933,11 @@ async def upload_tournament(tournament: models.Tournament, rounds: int) -> None:
             event_id = None
             # http POST https://www.vekn.net/api/vekn/event
             # "Authorization: Bearer <TOKEN>"
+            # "Vekn-Id: <USER_VEKN>"
             # -f name=<NAME> type=<TYPE> <...>
             async with session.post(
                 "https://www.vekn.net/api/vekn/event",
-                headers={"Authorization": f"Bearer {token}"},
+                headers={"Authorization": f"Bearer {token}", "Vekn-Id": user_vekn},
                 data=data,
             ) as response:
                 response.raise_for_status()
@@ -984,3 +1001,31 @@ def to_archondata(tournament: models.Tournament) -> str:
             f"§{player.result.tp}§{player.toss}§{ratings[player.uid].rating_points}§"
         )
     return ret
+
+
+async def create_member(member: models.Member) -> None:
+    try:
+        first, last = member.name.split(" ", 1)
+    except ValueError:
+        first = member.name
+        last = "N/A"
+    async with aiohttp.ClientSession() as session:
+        token = await get_token(session)
+        async with session.post(
+            "https://www.vekn.net/api/vekn/registry",
+            headers={"Authorization": f"Bearer {token}"},
+            params={
+                "veknid": member.vekn,
+                "firstname": first,
+                "lastname": last,
+                "email": member.email or f"{first}@example.com",
+                "country": member.country,
+                "state": "",
+                "city": member.city or "N/A",
+            },
+        ) as response:
+            response.raise_for_status()
+            result = await response.json()
+            result = result["data"]
+            if result["code"] != 200:
+                raise RuntimeError(f"Failed to create member on vekn.net: {result}")
